@@ -21,28 +21,30 @@
 #include "phrasebook.h"
 #include "phrasebookparser.h"
 
-#include <QtGui/QPainter>
-#include <QtCore/QFile>
-#include <QtCore/QRegExp>
-#include <QtCore/QStack>
-#include <QtCore/QTextStream>
-#include <QtGui/QMenu>
-#include <QtXml/QXmlInputSource>
-#include <QtXml/QXmlSimpleReader>
+#include <QAction>
+#include <QBuffer>
+#include <QFile>
+#include <QFileDialog>
+#include <QFontDatabase>
+#include <QPainter>
+#include <QRegExp>
+#include <QStack>
+#include <QTemporaryFile>
+#include <QTextStream>
+#include <QUrl>
+#include <QXmlInputSource>
+#include <QXmlSimpleReader>
 
-#include <kactionmenu.h>
-#include <kactioncollection.h>
-#include <klocale.h>
-#include <kaction.h>
-#include <kdesktopfile.h>
-#include <kmenu.h>
-#include <kstandarddirs.h>
-#include <ktoolbar.h>
-#include <ktemporaryfile.h>
-#include <kio/netaccess.h>
-#include <kfiledialog.h>
-#include <kmessagebox.h>
-#include <kglobalsettings.h>
+#include <KActionMenu>
+#include <KActionCollection>
+#include <KDesktopFile>
+#include <KIO/StoredTransferJob>
+#include <KLocalizedString>
+#include <QMenu>
+#include <KMessageBox>
+#include <KToolBar>
+
+#include <QDebug>
 
 Phrase::Phrase()
 {
@@ -124,7 +126,7 @@ void PhraseBook::print(QPrinter *pPrinter)
     int x = size.x();
     int y = size.y();
     int w = size.width();
-    printpainter.setFont(QFont(KGlobalSettings::generalFont().family(), 12));
+    printpainter.setFont(QFont(QFontDatabase::systemFont(QFontDatabase::GeneralFont).family(), 12));
     QFontMetrics metrics = printpainter.fontMetrics();
 
     PhraseBookEntryList::iterator it;
@@ -159,7 +161,7 @@ bool PhraseBook::decode(QXmlInputSource &source)
 {
     PhraseBookParser parser;
     QXmlSimpleReader reader;
-    reader.setFeature(QLatin1String("http://trolltech.com/xml/features/report-start-end-entity"), true);
+    reader.setFeature(QStringLiteral("http://trolltech.com/xml/features/report-start-end-entity"), true);
     reader.setContentHandler(&parser);
 
     if (reader.parse(source)) {
@@ -188,7 +190,7 @@ QByteArray encodeString(const QString str)
 QString PhraseBook::encode()
 {
     QString result;
-    result  = QLatin1String("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+    result  = QStringLiteral("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
     result += QLatin1String("<!DOCTYPE phrasebook>\n");
     result += QLatin1String("<phrasebook>\n");
 
@@ -207,11 +209,11 @@ QString PhraseBook::encode()
 
         if ((*it).isPhrase()) {
             Phrase phrase = (*it).getPhrase();
-            result += QLatin1String("<phrase shortcut=\"") + QLatin1String(encodeString(phrase.getShortcut()));
-            result += QLatin1String("\">") + QLatin1String(encodeString(phrase.getPhrase())) + QLatin1String("</phrase>\n");
+            result += QStringLiteral("<phrase shortcut=\"") + QLatin1String(encodeString(phrase.getShortcut()));
+            result += QStringLiteral("\">") + QLatin1String(encodeString(phrase.getPhrase())) + QStringLiteral("</phrase>\n");
         } else {
             Phrase phrase = (*it).getPhrase();
-            result += QLatin1String("<phrasebook name=\"") + QLatin1String(encodeString(phrase.getPhrase())) + QLatin1String("\">\n");
+            result += QStringLiteral("<phrasebook name=\"") + QLatin1String(encodeString(phrase.getPhrase())) + QStringLiteral("\">\n");
             level++;
         }
     }
@@ -235,9 +237,9 @@ QStringList PhraseBook::toStringList()
     return result;
 }
 
-bool PhraseBook::save(const KUrl &url)
+bool PhraseBook::save(const QUrl &url)
 {
-    QRegExp pattern(QLatin1String("*.phrasebook"), Qt::CaseSensitive, QRegExp::Wildcard);
+    QRegExp pattern(QStringLiteral("*.phrasebook"), Qt::CaseSensitive, QRegExp::Wildcard);
     return save(url, pattern.exactMatch(url.fileName()));
 }
 
@@ -247,10 +249,10 @@ void PhraseBook::save(QTextStream &stream, bool asPhrasebook)
     if (asPhrasebook)
         stream << encode();
     else
-        stream << toStringList().join(QLatin1String("\n"));
+        stream << toStringList().join(QStringLiteral("\n"));
 }
 
-bool PhraseBook::save(const KUrl &url, bool asPhrasebook)
+bool PhraseBook::save(const QUrl &url, bool asPhrasebook)
 {
     if (url.isLocalFile()) {
         QFile file(url.path());
@@ -266,70 +268,69 @@ bool PhraseBook::save(const KUrl &url, bool asPhrasebook)
         else
             return true;
     } else {
-        KTemporaryFile tempFile;
-        tempFile.open();
-
-        QTextStream ts(&tempFile);
+        QByteArray data;
+        QTextStream ts(&data);
         save(ts, asPhrasebook);
         ts.flush();
 
-        return KIO::NetAccess::upload(tempFile.fileName(), url, 0L);
+        KIO::StoredTransferJob *uploadJob = KIO::storedPut(data, url, -1);
+        return uploadJob->exec();
     }
 }
 
-int PhraseBook::save(QWidget *parent, const QString &title, KUrl &url, bool phrasebookFirst)
+int PhraseBook::save(QWidget *parent, const QString &title, QUrl &url, bool phrasebookFirst)
 {
     // KFileDialog::getSaveUrl(...) is not useful here as we need
     // to know the requested file type.
 
     QString filters;
     if (phrasebookFirst)
-        filters = i18n("*.phrasebook|Phrase Books (*.phrasebook)\n*.txt|Plain Text Files (*.txt)\n*|All Files");
+        filters = i18n("Phrase Books (*.phrasebook);;Plain Text Files (*.txt);;All Files (*)");
     else
-        filters = i18n("*.txt|Plain Text Files (*.txt)\n*.phrasebook|Phrase Books (*.phrasebook)\n*|All Files");
+        filters = i18n("Plain Text Files (*.txt);;Phrase Books (*.phrasebook);;All Files (*)");
 
-    QString empty;
-    KFileDialog fdlg(empty, filters, parent);
-    fdlg.setCaption(title);
-    fdlg.setOperationMode(KFileDialog::Saving);
+    QFileDialog fdlg(parent, title, QString(), filters);
+    fdlg.setAcceptMode(QFileDialog::AcceptSave);
 
-    if (fdlg.exec() != QDialog::Accepted) {
+    if (fdlg.exec() != QDialog::Accepted || fdlg.selectedUrls().size() < 1) {
         return 0;
     }
 
-    url = fdlg.selectedUrl();
+    url = fdlg.selectedUrls().at(0);
 
     if (url.isEmpty() || !url.isValid()) {
         return -1;
     }
 
-    if (KIO::NetAccess::exists(url, KIO::NetAccess::DestinationSide, 0L)) {
-        if (KMessageBox::warningContinueCancel(0, QString(QLatin1String("<qt>%1</qt>")).arg(i18n("The file %1 already exists. "
+    if (QFile::exists(url.toLocalFile())) {
+        if (KMessageBox::warningContinueCancel(0, QStringLiteral("<qt>%1</qt>").arg(i18n("The file %1 already exists. "
                                                "Do you want to overwrite it?", url.url())), i18n("File Exists"), KGuiItem(i18n("&Overwrite"))) == KMessageBox::Cancel) {
             return 0;
         }
     }
 
     bool result;
-    if (fdlg.currentFilter() == QLatin1String("*.phrasebook")) {
+    if (fdlg.selectedNameFilter() == QLatin1String("*.phrasebook")) {
         if (url.fileName(0).contains(QLatin1Char('.')) == 0) {
-            url.setFileName(url.fileName(0) + QLatin1String(".phrasebook"));
-        } else if (url.fileName(0).right(11).contains(QLatin1String(".phrasebook"), Qt::CaseInsensitive) == 0) {
-            int filetype = KMessageBox::questionYesNoCancel(0, QString(QLatin1String("<qt>%1</qt>")).arg(i18n("Your chosen filename <i>%1</i> has a different extension than <i>.phrasebook</i>. "
+            url = url.adjusted(QUrl::RemoveFilename);
+            url.setPath(url.path() + url.fileName(0) + QStringLiteral(".phrasebook"));
+        } else if (url.fileName(0).rightRef(11).contains(QLatin1String(".phrasebook"), Qt::CaseInsensitive) == 0) {
+            int filetype = KMessageBox::questionYesNoCancel(0, QStringLiteral("<qt>%1</qt>").arg(i18n("Your chosen filename <i>%1</i> has a different extension than <i>.phrasebook</i>. "
                            "Do you wish to add <i>.phrasebook</i> to the filename?", url.fileName())), i18n("File Extension"), KGuiItem(i18n("Add")), KGuiItem(i18n("Do Not Add")));
             if (filetype == KMessageBox::Cancel) {
                 return 0;
             }
             if (filetype == KMessageBox::Yes) {
-                url.setFileName(url.fileName(0) + QLatin1String(".phrasebook"));
+                url = url.adjusted(QUrl::RemoveFilename);
+                url.setPath(url.path() + url.fileName(0) + QStringLiteral(".phrasebook"));
             }
         }
         result = save(url, true);
-    } else if (fdlg.currentFilter() == QLatin1String("*.txt")) {
-        if (url.fileName(0).right(11).contains(QLatin1String(".phrasebook"), Qt::CaseInsensitive) == 0) {
+    } else if (fdlg.selectedNameFilter() == QLatin1String("*.txt")) {
+        if (url.fileName(0).rightRef(11).contains(QLatin1String(".phrasebook"), Qt::CaseInsensitive) == 0) {
             result = save(url, false);
         } else {
-            int filetype = KMessageBox::questionYesNoCancel(0, QString(QLatin1String("<qt>%1</qt>")).arg(i18n("Your chosen filename <i>%1</i> has the extension <i>.phrasebook</i>. "
+            int filetype = KMessageBox::questionYesNoCancel(0, QStringLiteral("<qt>%1</qt>").arg(i18n("Your chosen filename <i>%1</i> has the extension <i>.phrasebook</i>. "
                            "Do you wish to save in phrasebook format?", url.fileName())), i18n("File Extension"), KGuiItem(i18n("As Phrasebook")), KGuiItem(i18n("As Plain Text")));
             if (filetype == KMessageBox::Cancel) {
                 return 0;
@@ -349,23 +350,15 @@ int PhraseBook::save(QWidget *parent, const QString &title, KUrl &url, bool phra
         return -1;
 }
 
-bool PhraseBook::open(const KUrl &url)
+bool PhraseBook::open(const QUrl &url)
 {
-    QString tempFile;
-    KUrl fileUrl = url;
-
-    QString protocol = fileUrl.protocol();
-    if (protocol.isEmpty() || protocol.isNull()) {
-        fileUrl.setProtocol(QLatin1String("file"));
-        fileUrl.setPath(url.url());
-    }
-
-    if (KIO::NetAccess::download(fileUrl, tempFile, 0L)) {
-        QStringList list = QStringList();
-
+    KIO::StoredTransferJob *downloadJob = KIO::storedGet(url);
+    if (downloadJob->exec()) {
         // First: try to load it as a normal phrase book
-        QFile file(tempFile);
-        QXmlInputSource source(&file);
+        QBuffer fileBuffer;
+        fileBuffer.setData(downloadJob->data());
+
+        QXmlInputSource source(&fileBuffer);
         bool error = !decode(source);
 
         // Second: if the file does not contain a phrase book, load it as
@@ -373,21 +366,15 @@ bool PhraseBook::open(const KUrl &url)
         if (error) {
             // Load each line of the plain text file as a new phrase
 
-            QFile file(tempFile);
-            if (file.open(QIODevice::ReadOnly)) {
-                QTextStream stream(&file);
+            QTextStream stream(&fileBuffer);
 
-                while (!stream.atEnd()) {
-                    QString s = stream.readLine();
-                    if (!(s.isNull() || s.isEmpty()))
-                        *this += PhraseBookEntry(Phrase(s, QLatin1String("")), 0, true);
-                }
-                file.close();
-                error = false;
-            } else
-                error = true;
+            while (!stream.atEnd()) {
+                QString s = stream.readLine();
+                if (!(s.isNull() || s.isEmpty()))
+                    *this += PhraseBookEntry(Phrase(s, QLatin1String("")), 0, true);
+            }
+            error = false;
         }
-        KIO::NetAccess::removeTempFile(tempFile);
 
         return !error;
     }
@@ -397,10 +384,19 @@ bool PhraseBook::open(const KUrl &url)
 StandardBookList PhraseBook::standardPhraseBooks()
 {
     // Get all the standard phrasebook filenames in bookPaths.
-    QStringList bookPaths = KGlobal::mainComponent().dirs()->findAllResources(
-                                "data", QLatin1String("kmouth/books/*.phrasebook"),
-                                KStandardDirs::Recursive |
-                                KStandardDirs::NoDuplicates);
+    QStringList bookPaths;
+    const QStringList dirs =
+        QStandardPaths::locateAll(QStandardPaths::AppDataLocation, QStringLiteral("books"), QStandardPaths::LocateDirectory);
+    Q_FOREACH (const QString &dir, dirs) {
+        const QStringList locales = QDir(dir).entryList(QDir::Dirs | QDir::NoDotAndDotDot);
+        Q_FOREACH (const QString &locale, locales) {
+            const QStringList fileNames =
+                QDir(dir + '/' + locale).entryList(QStringList() << QStringLiteral("*.phrasebook"));
+            Q_FOREACH (const QString &file, fileNames) {
+                bookPaths.append(dir + '/' + locale + '/' + file);
+            }
+        }
+    }
     QStringList bookNames;
     QMap<QString, StandardBook> bookMap;
     QStringList::iterator it;
@@ -408,7 +404,7 @@ StandardBookList PhraseBook::standardPhraseBooks()
     for (it = bookPaths.begin(); it != bookPaths.end(); ++it) {
         PhraseBook pbook;
         // Open the phrasebook.
-        if (pbook.open(KUrl(*it))) {
+        if (pbook.open(QUrl::fromLocalFile(*it))) {
             StandardBook book;
             book.name = (*pbook.begin()).getPhrase().getPhrase();
 
@@ -434,12 +430,12 @@ QString PhraseBook::displayPath(QString filename)
     QFileInfo file(filename);
     QString path = file.path();
     QString dispPath;
-    int position = path.indexOf(QLatin1String("/kmouth/books/")) + QString(QLatin1String("/kmouth/books/")).length();
+    int position = path.indexOf(QLatin1String("/kmouth/books/")) + QStringLiteral("/kmouth/books/").length();
 
     while (path.length() > position) {
         file.setFile(path);
 
-        KDesktopFile *dirDesc = new KDesktopFile("data", path + QLatin1String("/.directory"));
+        KDesktopFile *dirDesc = new KDesktopFile(QStandardPaths::GenericDataLocation, path + QStringLiteral("/.directory"));
         QString name = dirDesc->readName();
         delete dirDesc;
 
@@ -465,7 +461,7 @@ void PhraseBook::addToGUI(QMenu *popup, KToolBar *toolbar, KActionCollection *ph
         for (it = begin(); it != end(); ++it) {
             int newLevel = (*it).getLevel();
             while (newLevel > level) {
-                KActionMenu *menu = phrases->add<KActionMenu>(QLatin1String("phrasebook"));
+                KActionMenu *menu = phrases->add<KActionMenu>(QStringLiteral("phrasebook"));
                 menu->setDelayed(false);
                 if (parent == popup)
                     toolbar->addAction(menu);
@@ -482,7 +478,7 @@ void PhraseBook::addToGUI(QMenu *popup, KToolBar *toolbar, KActionCollection *ph
             }
             if ((*it).isPhrase()) {
                 Phrase phrase = (*it).getPhrase();
-                KAction *action = new PhraseAction(phrase.getPhrase(),
+                QAction *action = new PhraseAction(phrase.getPhrase(),
                                                    phrase.getShortcut(), receiver, slot, phrases);
                 if (parent == popup)
                     toolbar->addAction(action);
@@ -490,7 +486,7 @@ void PhraseBook::addToGUI(QMenu *popup, KToolBar *toolbar, KActionCollection *ph
                     parent->addAction(action);
             } else {
                 Phrase phrase = (*it).getPhrase();
-                KActionMenu *menu = phrases->add<KActionMenu>(QLatin1String("phrasebook"));
+                KActionMenu *menu = phrases->add<KActionMenu>(QStringLiteral("phrasebook"));
                 menu->setText(phrase.getPhrase());
                 menu->setDelayed(false);
                 if (parent == popup)
